;##############################################################################
;
; Copyright (C) 2013, Michele Cappellari
; E-mail: cappellari_at_astro.ox.ac.uk
;
; Updated versions of the software are available from my web page
; http://purl.org/cappellari/software
;
; This software is provided as is without any warranty whatsoever.
; Permission to use, for non-commercial purposes is granted.
; Permission to modify for personal or internal use is granted,
; provided this copyright and disclaimer are included unchanged
; at the beginning of the file. All other rights are reserved.
;
;##############################################################################
;+
; NAME:
;   CAP_POLYFIT_2D
;
; PURPOSE:
;   Fit a bivariate polynomial of given DEGREE to a set of points
;   (X,Y,Z), assuming errors SIGZ in the Z variable only.
;
;   For example with DEGREE=1 this function fits a plane
;
;       z = a + b*x + c*y
;
;   while with DEGREE=2 the function fits a quadratic surface
;
;       z = a + b*x + c*x^2 + d*y + e*x*y + f*y^2
;
; CALLING SEQUENCE:
;   zfit = CAP_POLYFIT_2D(x, y, z, degree, SIGZ=sigz, COEFF=coeff)
;
; MODIFICATION HISTORY:
;   V1.0: Michele Cappellari, Oxford, 30 October 2013
;-
;##############################################################################
function cap_polyfit_2d, x, y, z, degree, $
    SIGZ=sigz, WEIGHTS=weights, COEFF=coeff
COMPILE_OPT IDL2    
ON_ERROR, 2

    sw = ~n_elements(weights) ? $ 
        (~n_elements(sigz) ? 1d : 1d/sigz) : sqrt(weights)
    
    np = (degree+1)*(degree+2) / 2
    a = (c = dblarr(np,n_elements(x)))
    k = 0
    for i=0,degree do $
        for j=0,degree-i do begin
            c[k,*] = x^j * y^i
            a[k,*] = c[k,*]*sw
            k++
        endfor
    coeff = LA_LEAST_SQUARES(a,z*sw)
    
    return, reform(c ## coeff)
end
;----------------------------------------------------------------------------------
